package Hawat::Controller::Alert;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Data::Dumper;
use Mentat::Core qw(:c_hawat);

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Hawat::Controller::Secure'; }

__PACKAGE__->config(
    class       => 'Hawat::Controller::Alert',
    constructor => 'new',
);

use constant SELECTORS_CACHE_REWRITE => 0; # Grow up mongodb performance !!!
use constant SELECTORS_STAT_ONLY     => 1; # Grow up mongodb performance !!!

=head1 NAME

Hawat::Controller::Alert - Catalyst controller for single alert operations

=head1 DESCRIPTION

Catalyst controller for handling actions related to single event alerts

=head1 ACTIONS

=cut

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

=head2 index

Default controller action, redirects to the 'list' action

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;
    $c->response->redirect($c->uri_for($self->action_for('list')));
}

=head2 list

View the list of all events stored in the database

=cut

sub list :Local :Args(0) {
    my ($self, $c) = @_;
    
    ## Store the back link to this request
    $c->session->{backlink} = $c->req->uri;
    
    # Determine which type of query to run based on the user role
    #my $quick_find = ($c->check_roles('developer'))?0:1;
    my $quick_find = 1;
    
    # Determine the type of the action user wants to execute
    #my $subaction = $c->request->params->{action} || 'search';
    my $subaction = $c->request->params->{action} || '';
    $subaction = lc($subaction); $subaction =~ s/\s//g;
    
    # Get the form data
    my $form = {
        'advanced'    => $c->request->params->{advanced},
        'raw_filter'  => $c->request->params->{raw_filter},
        'action'      => $c->request->params->{action},
        'query_name'  => $c->request->params->{query_name},
        'source'      => $c->request->params->{source},
        'logop'       => $c->request->params->{logop},
        'target'      => $c->request->params->{target},
        'datefrom'    => $c->request->params->{datefrom},
        'dateto'      => $c->request->params->{dateto},
        'detector'    => $c->request->params->{detector},
        'category'    => $c->request->params->{category},
        'abuse'       => $c->request->params->{abuse},
    };
    $form->{source}     = $self->clean_form_value($form->{source})     if defined $form->{source};
    $form->{target}     = $self->clean_form_value($form->{target})     if defined $form->{target};
    $form->{datefrom}   = $self->clean_form_value($form->{datefrom})   if defined $form->{datefrom};
    $form->{dateto}     = $self->clean_form_value($form->{dateto})     if defined $form->{dateto};
    $form->{abuse}      = $self->clean_form_value($form->{abuse})      if defined $form->{abuse};
    $form->{query_name} = $self->clean_form_value($form->{query_name}) if defined $form->{query_name};
    
    # Default lower time boudary
    $form->{datefrom} = $c->time_before('1week', 1) unless $form->{datefrom};
    $c->stash(datefrom => $form->{datefrom});
    
    $c->forward('setQueryData');
    $c->forward('setSelectData');

    # Initialize the searching condition, if the form was submitted
    if ($form->{action}) {
        $form = Hawat->model(HAWAT_MODEL_QUERY())->process_form($form);
        #$c->log->info("Calculated raw filter: ".$form->{raw_filter});
    }

    if ($c->request->params->{user_query} or $c->request->params->{group_query}) {
        $subaction = 'load';
        $c->forward('loadQuery');
        $form = $c->stash->{loaded_query};
    }

    # Perform searching
    if ($subaction eq 'search') {
        # Only admin may use advanced form
        $c->detach('access_denied') if ($form->{advanced} and not $c->check_roles('admin'));

        
        # Configure and forward to '/dbitem/find' action
        my $conf = $self->_dbicnf(
            advanced => $form,
            quick    => $quick_find,
            limit    => $c->cfg('result_limit'),
            sortby   => 'dt',
            sort     => 'desc',
        );
        $c->forward(HAWAT_DBI_FIND, [$conf, $c]);
        
        my $iu = shift @{$c->stash->{alerts_more}};
        $self->store_query($c, 'Find alerts index usage', Dumper($iu)) if $c->cfg('devel');
        
        #my $fr = $c->stash->{alerts};
        #$c->stash(frs => $c->model('Alerts')->calculate_frs($fr, $c));
    }
    # Save the query for later use
    elsif ($subaction eq 'save') {
        $c->forward('saveQuery', [$form]);
        
        foreach my $k (keys %$form) {
            $c->stash($k => $form->{$k}) if defined $form->{$k};
        }

        $c->forward('setQueryData');
    }
    # Switch to simple form
    elsif ($subaction eq 'gosimple') {
        $form->{advanced} = 0;
        foreach my $k (keys %$form) {
            $c->stash($k => $form->{$k}) if defined $form->{$k};
        }
    }
    # Switch to advanced form
    elsif ($subaction eq 'goadvanced') {
        $form->{advanced} = 1;
        foreach my $k (keys %$form) {
            $c->stash($k => $form->{$k}) if defined $form->{$k};
        }
    }
    
    $c->stash->{selected_detector} = $c->model(HAWAT_MODEL_QUERY())->getSelected($form->{detector});
    $c->stash->{selected_category} = $c->model(HAWAT_MODEL_QUERY())->getSelected($form->{category});
    $c->stash->{advanced}          = $form->{advanced};

    # Choose the template to fill
    $c->stash(template => 'alert/list.html.tt');

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the devel switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Secure for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 view

View the details of one particular event alert

=cut

#sub view :Local :Args(1) { # Switch to let the framework handle the error reporting
sub view :Local {
    my ($self, $c, $id) = @_;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH,[$conf, $id]);

    # Move the alert tree to better location in stash
    if ($c->stash->{alert_count}) {
        $c->stash(alert_raw => shift $c->stash->{alert_more});

        my $template_id = $c->stash->{alert}->path_value('_CESNET/EventTemplate');
        if ($template_id) {
            my ($event, $ecount, $eduration) = $c->model('Events')->fetch($template_id);
            $c->stash(event => $event);
        }

        # Attempt to generate email bodies
        my $email = $c->model('Alerts')->generate_email($c->stash->{alert});
        $c->stash(example_email => $email) if $email;
    }

    # Store the back link to this request
    $c->session->{bl_alerts} = $c->req->uri;

    # Optionally dump interesting variables - dependent on the devel switch
    $self->dump_variable($c, 'Alert raw', $c->stash->{alert_raw}) if $c->cfg('devel');
    $self->dump_variable($c, 'Stash', $c->stash) if $c->cfg('devel');

    # Choose the template to fill
    $c->stash(template => 'alert/view.html.tt');
}

=head2 delete

Delete one particular alert

=cut

#sub delete :Local :Args(1) {# Switch to let the framework handle the error reporting
sub delete :Local {
    my ($self, $c, $id, $confirm) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{bl_alerts} || $c->uri_for('/alert');

    # Configure and forward to '/dbitem/delete' action
    my $conf = $self->_dbicnf(
        backlink   => $backlink,
        backlink_d => $c->uri_for('/alert'),
    );
    $c->forward(HAWAT_DBI_DELETE, [$conf, $id, $confirm]);
}

=head2 data

Get the alert data

=cut

#sub data :Local :Args(1) {# Switch to let the framework handle the error reporting
sub data :Local {
    my ($self, $c, $id) = @_;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH,[$conf, $id]);

    if ($c->stash->{alert_count}) {
        my $alert = $c->stash->{alert};
        my $fn    = "alert.".$alert->class().".json";

        $c->res->header('Content-Disposition', qq[attachment; filename="$fn"]);
        $c->res->content_type('application/json');
        $c->res->body($alert->to_string(1));

    }
    else {
        $c->detach(HAWAT_ERROR_HANDLER, [message => "Unknown alert ID '$id'", menu => $self->name]);
        return; # Only for readability, detach() works through DIE
    }
}

=head2 report

Send the alert as report

=cut

#sub data :Local :Args(1) {# Switch to let the framework handle the error reporting
sub report :Local {
    my ($self, $c, $id) = @_;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH,[$conf, $id]);

    if ($c->stash->{alert_count}) {
        my $alert = $c->stash->{alert};
        my $error = $c->model('Alerts')->report($alert);

        if ($error) {

        }
        else {

        }
    }
    else {
        $c->detach(HAWAT_ERROR_HANDLER, [message => "Unknown alert ID '$id'", menu => $self->name]);
        return; # Only for readability, detach() works through DIE
    }
}

#-------------------------------------------------------------------------------

sub saveQuery :Private {
    my ($self, $c, $form) = @_;

    unless ($form->{query_name}) {
        $self->flash_error($c, "Name of the query is mandatory.");
        return;
    }

    # Process the form into query object
    my $query = Hawat->model(HAWAT_MODEL_QUERY())->process_form($form);

    # First check, if the query already exists
    my $q;
    # Fetch group related query
    if (length ($form->{query_group}) > 0) {
        $q = $c->model(HAWAT_MODEL_GROUPS())->getQuery($form->{query_group}, $query->{HAWAT_ATTR_MID()});
    }
    # Fetch user related query
    else {
        $q = $c->model(HAWAT_MODEL_USERS())->getQuery($c->get_current_user_id(), $query->{HAWAT_ATTR_MID()});
    }
    # And check the existence
    if ($q) {
        $self->flash_error($c, "Query with name '".$query->{HAWAT_ATTR_MID()}."' already exists.");
        return;
    }

    my $result;
    if (length ($form->{HAWAT_FORM_GROUP()}) > 0)
    {
        $result = $c->model(HAWAT_MODEL_GROUPS())->addQuery($form->{HAWAT_FORM_GROUP()}, $query);
    }
    else {
        $result = $c->model(HAWAT_MODEL_USERS())->addQuery($c->get_current_user_id(), $query);
    }

    if (defined $result->{err}) {
        $self->flash_error($c, "Unable to insert query: ".$result->{err}.".");
        return;
    }
    else {
        $self->flash_success($c, "Query has been inserted");
    }
}

sub loadQuery :Private {
    my ($self, $c) = @_;

    my ($query, $qid);
    if ($c->request->params->{user_query}) {
        $qid = $c->request->params->{user_query};
        #$query = $c->model(HAWAT_MODEL_USERS())->getQuery($c->get_current_user_id(), $c->request->params->{user_query});
        $query = $c->stash->{'list_queries_user'}->{$c->request->params->{user_query}};
    }
    elsif ($c->request->params->{group_query}) {
        $qid = $c->request->params->{group_query};
        #$query = $c->model(HAWAT_MODEL_GROUPS())->getQuery($c->request->params->{group_id}, $c->request->params->{group_query});
        $query = $c->stash->{'list_queries_groups'}->{$c->request->params->{group_id}}->{$c->request->params->{group_query}};
    }

    unless ($query) {
        $self->flash_error($c, "Query with ID '$qid' does not exist.");
        return;
    }

    foreach my $k (keys %$query) {
        $c->stash($k => $query->{$k}) if defined $query->{$k};
    }

    if ($query->{advanced} == 1 and not $c->request->params->{advanced}) {
        $c->request->params->{advanced} = 1;
    }

    $c->stash(loaded_query => $query);
    $self->flash_success($c, "Query '$qid' has been loaded.");
}

sub setQueryData :Private {
    my ($self, $c) = @_;

    # Get list of user and group queries, but do not stop rendering the page
    my ($user_queries, $group_queries);
    eval {
        $user_queries  = $c->model(HAWAT_MODEL_USERS())->getQueries($c->get_current_user_id());
        $group_queries = $c->model(HAWAT_MODEL_USERS())->getGroupQueries($c->get_current_user_id());

        # Put the retrieved data to stash
        $c->stash->{'list_queries_user'}   = $user_queries;
        $c->stash->{'list_queries_groups'} = $group_queries;
    };
    if ($@) { $self->flash_error($c, $@); }
}

sub setSelectData :Private {
    my ($self, $c) = @_;

    my $selectors;
    eval {
        $selectors = $c->model(HAWAT_MODEL_ALERTS())->get_item_sets(SELECTORS_CACHE_REWRITE(), SELECTORS_STAT_ONLY());

        # Put the retrieved data to stash
        #$c->stash->{'select_detector'} = $selectors->{Mentat::Core::KEY_CACHE_DETECTORS()};
        #$c->stash->{'select_category'} = $selectors->{Mentat::Core::KEY_CACHE_CATEGORIES()};
        $c->stash->{'select_detector'} = $selectors->{'itemset-stat-nodenames'};
        $c->stash->{'select_category'} = $selectors->{'itemset-stat-categories'};

        $c->stash->{'select_group'} = $c->model(HAWAT_MODEL_GROUPS())->listManagedGroups($c->get_current_user_id());
    };
    if ($@) { $self->flash_error($c, $@); }
}

# _dbicnf(%)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _dbicnf(%)
{
    my ($self, %cnf_a) = @_;
    my $cnf = {
            controller => 'Alert',
            model      => HAWAT_MODEL_ALERTS(),
            items      => 'Alerts',
            item       => 'Alert',
            menu       => 'Alerts',
        };
    foreach my $k (keys %cnf_a) {
        $cnf->{$k} = $cnf_a{$k};
    }
    return $cnf;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
