package Hawat::Controller::Filter;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Data::Dumper;
use File::Slurp;
use Mentat::Core qw(:c_hawat);

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Hawat::Controller::Secure'; }

__PACKAGE__->config(
    class       => 'Hawat::Controller::Filter',
    constructor => 'new',
);

=head1 NAME

Hawat::Controller::Filter - Catalyst controller for filter management

=head1 DESCRIPTION

Catalyst controller for handling actions related to filter management.

=head1 ACTIONS

=cut

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

=head2 index

Default controller action, redirects to the 'list' action

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;
    $c->response->redirect($c->uri_for($self->action_for('list')));
}

=head2 list

View the list of all reports stored in the database

=cut

sub list :Local :Args(0) {
    my ($self, $c) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/find' action
    my $conf = $self->_dbicnf(
        operation => 'listFilters',
        limit  => $c->cfg('result_limit'),
        sortby => 'id',
        sort   => 'asc',
    );
    $c->forward(HAWAT_DBI_LIST, [$conf]);

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 remove

Remove multiple events

=cut

#sub remove :Local :Args(1) {# Switch to let the framework handle the error reporting
sub remove :Local {
    my ($self, $c, $id, $confirm) = @_;

    my ($success, $error) = $c->model('Groups')->removeFilters();

    $self->flash_success($c, $success) if $success;
    $self->flash_error($c, $error) if $error;

    $c->response->redirect($c->uri_for($c->controller('Filter')->action_for('list')));
}

=head2 import

Import events from external file

=cut

sub import :Local {
    my ($self, $c) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/filter');

    # Determine the type of the action
    my $action = $c->request->params->{action} ||  undef;

    # Handle operation confirmation - display the question page
    unless ($action) {
        $c->stash(template => 'inc/dialog_upload.html.tt',
                  title => 'Hawat - Filter import',
                  menu => 'filters',
                  controller => 'filter',
                  action => 'import',
                  label => 'Target email',
                  backlink => $backlink);
        $c->forward($c->view(HAWAT_VIEW_HTML)); return;
    }
    # Handle the operation cancelation
    elsif (lc($action) =~ 'cancel') {
        $self->flash_info($c, "Filter import was canceled.");
        $c->response->redirect($backlink); return;
    }

    if (my $upload = $c->request->upload('uploaded_file') ) {
        my $filename = $upload->filename;
        my $target   = "/tmp/$filename";

        unless ($upload->link_to($target) || $upload->copy_to($target)) {
            die "Failed to copy '$filename' to '$target': $!\n";
        }

        $c->log->info('IMPORT: File received');

        #$self->flash_success($c, "Upload: ".read_file($target));
        my ($result, $errors) = $c->model('Groups')->importFilters($target, $c);

        $c->log->info('IMPORT: Filters imported');

        foreach my $r (@$result) {
            $self->flash_success($c, $r);
        }
        foreach my $e (@$errors) {
            $self->flash_error($c, $e);
        }

        $c->log->info('IMPORT: File error displayed');
    }

    $c->response->redirect($c->uri_for($c->controller('Filter')->action_for('list')));
}

=head2 export

Export events to external file

=cut

sub export :Local {
    my ($self, $c) = @_;

    my $json = $c->model('Groups')->exportFilters();
    $c->response->body($json);
    $c->response->header('Content-Disposition' => 'attachment; filename="msgfilters.json"');
    $c->response->content_type('application/json; charset=utf-8');

    #$c->stash('events' => $json);
    #$c->forward($c->view(HAWAT_VIEW_JSON));
}

# _dbicnf(%)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _dbicnf(%)
{
    my ($self, %cnf_a) = @_;
    my $cnf = {
            controller => 'Filter',
            model      => HAWAT_MODEL_GROUPS(),
            items      => 'Filters',
            item       => 'Filter',
            menu       => 'admin',
        };
    foreach my $k (keys %cnf_a) {
        $cnf->{$k} = $cnf_a{$k};
    }
    return $cnf;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
