package Hawat::Controller::Runlog;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Data::Dumper;
use Mentat::Core qw(:c_hawat);

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Hawat::Controller::Secure'; }

__PACKAGE__->config(
    class       => 'Hawat::Controller::Runlog',
    constructor => 'new',
);

=head1 NAME

Hawat::Controller::Runlog - Catalyst controller for event reports

=head1 DESCRIPTION

Catalyst controller for handling actions related to event reports.

=head1 METHODS

=cut

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

=head2 index

Default controller action, redirects to the 'list' action

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;
    $c->response->redirect($c->uri_for($self->action_for('list')));
}

=head2 reporter_list

View the list of all reports stored in the database

=cut

sub reporter_list :Local :Args(0) {
    my ($self, $c) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/find' action
    my $conf = $self->_dbicnf(
        limit  => $c->cfg('result_limit'),
        sortby => 'ts',
        sort   => 'desc',
    );
    $c->forward(HAWAT_DBI_FIND, [$conf]);

    #my $fr = $c->stash->{runlogs};
    #$c->stash(frs => $c->model('Runlogs')->calculate_frs($fr, $c));

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the devel switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Secure for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 reporter_view

View the details of arbitrary log

=cut

#sub reporter_view :Local :Args(1) { # Switch to let the framework handle the error reporting
sub reporter_view :Local {
    my ($self, $c, $id) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id]);

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    $self->dump_variable($c, 'Conf', $conf) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details

#sub COMPONENT {
#   my ($class, $app, $args) = @_;
#   $args = $class->merge_config_hashes($class->config, $args);
#   #use Data::Dumper;
#   #$app->log->debug("X $class: " . Dumper($class->config));
#   #$app->log->debug("Y args  : " . Dumper($args));
#   #$app->log->debug("Z $app  : " . Dumper($app->config));
#   #$app->log->debug("MERGE XY: " . Dumper($args));
#   return $class->new($app, $args);
#}

# _dbicnf(%)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _dbicnf(%)
{
    my ($self, %cnf_a) = @_;
    my $cnf = {
            controller => 'Runlog',
            model      => 'Runlogs',
            items      => 'Runlogs',
            item       => 'Runlog',
        };
    foreach my $k (keys %cnf_a) {
        $cnf->{$k} = $cnf_a{$k};
    }
    return $cnf;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
